/*
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * (the "License"); you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://www.mozilla.org/MPL/>.
 * 
 * Software distributed under the License is distributed on an "AS IS" basis, WITHOUT
 * WARRANTY OF ANY KIND, either express or implied. See the License for the specific
 * language governing rights and limitations under the License.
 * 
 * The Original Code is the Venice Web Communities System.
 * 
 * The Initial Developer of the Original Code is Eric J. Bowersox <erbo@silcom.com>,
 * for Silverwrist Design Studios.  Portions created by Eric J. Bowersox are
 * Copyright (C) 2001 Eric J. Bowersox/Silverwrist Design Studios.  All Rights Reserved.
 * 
 * Contributor(s): 
 */
package com.silverwrist.venice.servlets.format;

import java.io.Writer;
import java.io.IOException;
import java.util.*;
import javax.servlet.*;
import com.silverwrist.util.StringUtil;
import com.silverwrist.venice.core.*;
import com.silverwrist.venice.except.*;

public class FindPostData implements JSPRender
{
  /*--------------------------------------------------------------------------------
   * Static data members
   *--------------------------------------------------------------------------------
   */

  // Attribute name for request attribute
  protected static final String ATTR_NAME = "com.silverwrist.venice.content.FindPostData";

  /*--------------------------------------------------------------------------------
   * Attributes
   *--------------------------------------------------------------------------------
   */

  private CommunityContext comm;
  private ConferenceContext conf;
  private TopicContext topic;
  private String term = "";
  private int offset = 0;
  private int find_count = 0;
  private List results = null;
  private int max_results = 0;

  /*--------------------------------------------------------------------------------
   * Constructor
   *--------------------------------------------------------------------------------
   */

  public FindPostData(CommunityContext comm, ConferenceContext conf, TopicContext topic)
  {
    this.comm = comm;
    this.conf = conf;
    this.topic = topic;

  } // end constructor

  /*--------------------------------------------------------------------------------
   * Internal static functions
   *--------------------------------------------------------------------------------
   */

  private static int getParamInt(ServletRequest request, String name, int default_val)
  {
    String str = request.getParameter(name);
    if (str==null)
      return -1;

    try
    { // parse the integer value
      return Integer.parseInt(str);

    } // end try
    catch (NumberFormatException nfe)
    { // in case of conversion error, fall through
    } // end catch

    return default_val;

  } // end getParamInt

  private static boolean isImageButtonClicked(ServletRequest request, String name)
  {
    String val = request.getParameter(name + ".x");
    return (val!=null);

  } // end isImageButtonClicked

  /*--------------------------------------------------------------------------------
   * External static functions
   *--------------------------------------------------------------------------------
   */

  public static FindPostData retrieve(ServletRequest request)
  {
    return (FindPostData)(request.getAttribute(ATTR_NAME));

  } // end retrieve

  /*--------------------------------------------------------------------------------
   * Implementations from interface VeniceContent
   *--------------------------------------------------------------------------------
   */

  public String getPageTitle(RenderData rdat)
  {
    return "Find Posts";

  } // end getPageTitle

  public String getPageQID()
  {
    return null;

  } // end getPageQID

  /*--------------------------------------------------------------------------------
   * Implementations from interface JSPRender
   *--------------------------------------------------------------------------------
   */

  public void store(ServletRequest request)
  {
    request.setAttribute(ATTR_NAME,this);

  } // end store

  public String getTargetJSPName()
  {
    return "findpost.jsp";

  } // end getTargetJSPName

  /*--------------------------------------------------------------------------------
   * External operations
   *--------------------------------------------------------------------------------
   */

  public final void doSearch(ServletRequest request, VeniceEngine engine) throws AccessError, DataException
  {
    max_results = engine.getStdNumSearchResults();

    // Retrieve the search term parameter.
    term = request.getParameter("term");
    if (term==null)
      term = "";

    // Retrieve the offset and find count parameters.
    offset = getParamInt(request,"ofs",0);
    find_count = getParamInt(request,"fcount",-1);

    // Adjust the search return offset based on the command button click.
    if (isImageButtonClicked(request,"search"))
      offset = 0;
    else if (isImageButtonClicked(request,"previous"))
    { // adjust the offset in the reverse direction
      offset -= max_results;
      if (offset<0)
	offset = 0;

    } // end else if
    else if (isImageButtonClicked(request,"next"))
      offset += max_results;  // go forwards instead
    else
      throw new InternalStateError("Unable to determine what action triggered the form.");

    if (topic!=null)
    { // search the topic
      results = topic.searchPosts(term,offset,max_results);
      if (find_count<0)
	find_count = topic.getSearchPostCount(term);

    } // end if
    else if (conf!=null)
    { // search the conference
      results = conf.searchPosts(term,offset,max_results);
      if (find_count<0)
	find_count = conf.getSearchPostCount(term);

    } // end else if
    else
    { // search the community
      results = comm.searchPosts(term,offset,max_results);
      if (find_count<0)
	find_count = comm.getSearchPostCount(term);

    } // end else

  } // end doSearch

  public final String getExtraParameters()
  {
    StringBuffer buf = new StringBuffer("<INPUT TYPE=\"HIDDEN\" NAME=\"sig\" VALUE=\"");
    buf.append(comm.getCommunityID()).append("\">");
    if (conf!=null)
    { // append the conference number parameter
      buf.append("\n<INPUT TYPE=\"HIDDEN\" NAME=\"conf\" VALUE=\"").append(conf.getConfID()).append("\">");
      if (topic!=null)
      { // append the topic number parameter
	buf.append("\n<INPUT TYPE=\"HIDDEN\" NAME=\"top\" VALUE=\"").append(topic.getTopicNumber());
	buf.append("\">");

      } // end if

    } // end if

    return buf.toString();

  } // end getExtraParameters

  public final String getReturnLink(RenderData rdat)
  {
    if (topic!=null)
      return "<A HREF=\"" + rdat.getEncodedServletPath("confdisp?sig=" + comm.getCommunityID()
						       + "&conf=" + conf.getConfID() + "&top="
						       + topic.getTopicNumber()) + "\">Return to Topic</A>";
    else if (conf!=null)
      return "<A HREF=\"" + rdat.getEncodedServletPath("confdisp?sig=" + comm.getCommunityID()
						       + "&conf=" + conf.getConfID())
	                  + "\">Return to Topic List</A>";
    else
      return "<A HREF=\"" + rdat.getEncodedServletPath("confops?sig=" + comm.getCommunityID())
	                  + "\">Return to Topic List</A>";
      
  } // end getReturnLink

  public final String getSubtitle()
  {
    if (topic!=null)
      return "Topic: " + topic.getName();
    else if (conf!=null)
      return "Conference: " + conf.getName();
    else
      return "Community: " + comm.getName();

  } // end getSubtitle

  public final String getTerm()
  {
    return term;

  } // end getTerm

  public final int getOffset()
  {
    return offset;

  } // end getOffset

  public final int getFindCount()
  {
    return find_count;

  } // end getFindCount

  public final List getResults()
  {
    return results;

  } // end getResults

  public final int getMaxResults()
  {
    return max_results;

  } // end getMaxResults

} // end class FindPostData
